//-----------------------------------------------------------
// Android SDL Sample App
//
// Copyright (c) 2012 Motorola Solutions, Inc.
//-----------------------------------------------------------

package com.motorola.sdl;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

import com.motorolasolutions.adc.decoder.BarCodeReader;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.AudioManager;
import android.media.ToneGenerator;
import android.os.Bundle;
import android.os.Environment;
import android.os.SystemClock;

import android.util.Log;
import android.view.KeyEvent;
import android.view.SurfaceHolder;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;

public class SE4500_DEMO extends Activity implements BarCodeReader.DecodeCallback,BarCodeReader.PictureCallback,
														SurfaceHolder.Callback
{

	// ------------------------------------------------------
	static final private boolean saveSnapshot = false; // true = save snapshot to file
	
	// -----------------------------------------------------
	// statics
	static SE4500_DEMO app = null;
	
	//states
	static final int STATE_IDLE 		= 0;
	static final int STATE_DECODE 		= 1;
	static final int STATE_HANDSFREE	= 2;	
	static final int STATE_PREVIEW		= 3;	//snapshot preview mode
	static final int STATE_SNAPSHOT		= 4;
	static final int STATE_VIDEO 		= 5;
	
	private boolean STATE_ISDECODING       = false;
	

	// system
	private ToneGenerator tg 		        = null;
	
    private ImageView image 				= null;	//snaphot image screen
	
	// BarCodeReader specifics
	private BarCodeReader bcr 		= null;

	private EditText tvData         = null;
	private boolean beepMode 		= true; 		// decode beep enable
	private int trigMode			= BarCodeReader.ParamVal.AUTO_AIM;
	private int state				= STATE_IDLE;
	private int decodes 			= 0;
	
	private int motionEvents 		= 0;
	private int modechgEvents 	    = 0;
	
	private int snapNum				= 0;		//saved snapshot #
	static long decode_start = 0;
	static long decode_end   = 0;
    public static final int KEY_SCAN = 111;

	static
	{
		System.loadLibrary("IAL");
		System.loadLibrary("SDL");
		System.loadLibrary("barcodereader");
	}
	
	// ------------------------------------------------------
	public SE4500_DEMO()
	{
		app = this;
	}

	// ------------------------------------------------------
	// Called with the activity is first created.
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		mainScreen();
		
		// sound
		tg = new ToneGenerator(AudioManager.STREAM_MUSIC, ToneGenerator.MAX_VOLUME);
	}
	
	//-----------------------------------------------------
	@Override
	protected void onPause()
	{
		super.onPause();
		if (bcr != null)
		{
			setIdle();			
			bcr.release();
			bcr = null;
		}
	}

	// ------------------------------------------------------
	// Called when the activity is about to start interacting with the user.
	@Override
	protected void onResume()
	{
		super.onResume();
		state = STATE_IDLE;

		try
		{
			bcr = BarCodeReader.open(0);
			if (bcr == null)
			{
				return;
			}
			bcr.setDecodeCallback(this);
		}
		catch (Exception e)
		{
		}
		doSetParam(22,2);
		doSetParam(23,40);
		doSetParam(BarCodeReader.ParamNum.QR_INVERSE,BarCodeReader.ParamVal.INVERSE_AUTOD);
		doSetParam(BarCodeReader.ParamNum.DATAMATRIX_INVERSE,BarCodeReader.ParamVal.INVERSE_AUTOD);
	}

// === Android UI methods =======================================
	//-----------------------------------------------------
	// create main screen
	private void mainScreen()
	{
		setContentView(R.layout.image);		// Inflate our UI from its XML layout description.

		// Hook up button presses to the appropriate event handler.
		((Button) findViewById(R.id.buttonDec)).setOnClickListener(mDecodeListener);
		((Button) findViewById(R.id.buttonSnap)).setOnClickListener(mSnapListener);
		
		// ui items
		tvData = (EditText) findViewById(R.id.textDecode);
	}
	//-----------------------------------------------------	
	// create snapshot image screen
	private void snapScreen(Bitmap bmSnap)
	{
		image = (ImageView) findViewById(R.id.snap_image);		
		
		if (bmSnap != null)
			image.setImageBitmap(bmSnap);
	}
	
	//-----------------------------------------------------
	// SurfaceHolder callbacks	
	public void surfaceCreated(SurfaceHolder holder)
	{
		try
		{
   		if (state == STATE_PREVIEW)
   		{
				bcr.setPreviewDisplay(holder);
				bcr.startViewFinder();			//snapshot with preview mode
   		}
   		else //must be video	
   		{
				bcr.setPreviewDisplay(holder);
				bcr.startVideoCapture(null);				
   		}
		}
		catch (IOException e)
		{
		}
	}

	//-----------------------------------------------------
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height)
	{
	}

	//-----------------------------------------------------
	public void surfaceDestroyed(SurfaceHolder holder)
	{
	}
	
	// ------------------------------------------------------
	// callback for decode button press
	OnClickListener mDecodeListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			decode_start = SystemClock.elapsedRealtime();
			doDecode();
		}
	};

	// ------------------------------------------------------
	// callback for snapshot button press
	OnClickListener mSnapListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			doSnap();
		}
	};
	
	// ------------------------------------------------------	
	// callback for take-picture button on snap-preview screen
	OnClickListener mTakePicListener = new OnClickListener()
	{
		public void onClick(View v)
		{
			doSnap1();
		}
	};
	
	// ----------------------------------------
	// display status string
	private void dspData(String s)
	{
		tvData.setText(s);
	}

	private void beep()
	{
		if (tg != null)
			tg.startTone(ToneGenerator.TONE_CDMA_NETWORK_BUSY_ONE_SHOT);
	}

	
	// ----------------------------------------	
	private boolean isHandsFree()
	{
		return (trigMode == BarCodeReader.ParamVal.HANDSFREE);
	}

	// ----------------------------------------	
	private boolean isAutoAim()
	{
		return (trigMode == BarCodeReader.ParamVal.AUTO_AIM);		
	}

	// ----------------------------------------
	// reset Level trigger mode
	void resetTrigger()
	{
		doSetParam(BarCodeReader.ParamNum.PRIM_TRIG_MODE, BarCodeReader.ParamVal.LEVEL); 
		trigMode = BarCodeReader.ParamVal.LEVEL;		
	}
	
	// ----------------------------------------
	// set param
	private int doSetParam(int num, int val)
	{
		String s= "";
		int ret = bcr.setParameter(num, val);
		if (ret != BarCodeReader.BCR_ERROR)
		{
			if (num == BarCodeReader.ParamNum.PRIM_TRIG_MODE)
			{
				trigMode = val;
				if (val == BarCodeReader.ParamVal.HANDSFREE)
				{
					s = "HandsFree";
				}
				else if (val == BarCodeReader.ParamVal.AUTO_AIM)
				{
					s = "AutoAim";
					ret = bcr.startHandsFreeDecode(BarCodeReader.ParamVal.AUTO_AIM);
					if (ret != BarCodeReader.BCR_SUCCESS)
					{
					}
				}
				else if (val == BarCodeReader.ParamVal.LEVEL)
				{
					s = "Level";
				}
			}
		}
		else
			s = " FAILED (" + ret +")";
			
		return ret;
	}
	
	// ----------------------------------------
	// start a decode session
	private void doDecode()
	{
		if (setIdle() != STATE_IDLE)		
			return;
		
		dspData("");		
		state = STATE_DECODE;			
		bcr.startDecode(); // start decode (callback gets results)
	}
	// ----------------------------------------
	// BarCodeReader.DecodeCallback override
	public void onDecodeComplete(int symbology, int length, byte[] data, BarCodeReader reader)
	{
		if (state == STATE_DECODE)
			state = STATE_IDLE;
		
		if (length > 0)
		{
			decode_end = SystemClock.elapsedRealtime();
			Log.d("#########", "################time:" + (decode_end - decode_start));
			if (isHandsFree()==false && isAutoAim()==false)
				bcr.stopDecode();

			++decodes;
			
			if (beepMode)
				beep();
			
			if (symbology == 0x99)	//type 99?
			{
				symbology = data[0];
				int n = data[1];
				int s = 2;
				int d = 0;
				int len = 0;
				byte d99[] = new byte[data.length];					
				for (int i=0; i<n; ++i)
				{
					s += 2;
					len = data[s++];
					System.arraycopy(data, s, d99, d, len);
					s += len;
					d += len;
				}
				d99[d] = 0;
				data = d99;
			}

			dspData(new String(data,0,length));
			//doSnap();
		}
      else	// no-decode
      {
      	switch (length)
      	{
      	case BarCodeReader.DECODE_STATUS_TIMEOUT:
      		break;
      		
      	case BarCodeReader.DECODE_STATUS_CANCELED:
      		break;
      		
     		case BarCodeReader.DECODE_STATUS_ERROR:
      	default:
      		break;
      	}
      }
	  STATE_ISDECODING = false;
	}
	
	// ----------------------------------------
	// start a snap/preview session
	private void doSnap()
	{
		if (setIdle() != STATE_IDLE)		
			return;

		resetTrigger();
		//dspData("");		
		state = STATE_SNAPSHOT;
		snapScreen(null);
		bcr.takePicture(app);
		
	}

	// ----------------------------------------
	// take snapshot   
	private void doSnap1()
	{
		if (state == STATE_PREVIEW)
		{
			bcr.stopPreview();
			state = STATE_SNAPSHOT;
		}
		if (state == STATE_SNAPSHOT)
		{
			snapScreen(null);
			bcr.takePicture(app);
		}
		else //unexpected state - reset mode
		{
			setIdle();
			mainScreen();
		}
	}

	// ----------------------------------------
	public void onPictureTaken(int format, int width, int height, byte[] abData, BarCodeReader reader)
	{
		if (state != STATE_SNAPSHOT || image == null)
			return;			

		// display snapshot		
		Bitmap bmSnap = BitmapFactory.decodeByteArray(abData, 0, abData.length);
		if (bmSnap != null)
			image.setImageBitmap(bmSnap);
	
		// Save snapshot to the SD card
		if (saveSnapshot)
		{
			String snapFmt = "bin";			
			switch (bcr.getNumParameter(BarCodeReader.ParamNum.IMG_FILE_FORMAT))
			{
			case BarCodeReader.ParamVal.IMG_FORMAT_BMP:
				snapFmt = "bmp";
				break;
			
			case BarCodeReader.ParamVal.IMG_FORMAT_JPEG:
				snapFmt = "jpg";				
				break;
			
			case BarCodeReader.ParamVal.IMG_FORMAT_TIFF:
				snapFmt = "tif";	
				break;
			}
			
			File filFSpec = null;
			try
			{
				String strFile = String.format("se4500_img_%d.%s", snapNum, snapFmt);
				File filRoot = Environment.getExternalStorageDirectory();
				File filPath = new File(filRoot.getAbsolutePath() + "/DCIM/Camera");
				filPath.mkdirs();
				filFSpec = new File(filPath, strFile);
				FileOutputStream fos = new FileOutputStream(filFSpec);
				fos.write(abData);
				fos.close();
				
				++snapNum;
			}
			catch (Throwable thrw)
			{
			}
		}
	}
	
	//------------------------------------------	
	private int setIdle()
	{
		int prevState = state;
		int ret = prevState;		//for states taking time to chg/end
		
		state = STATE_IDLE;
		switch (prevState)
		{
		case STATE_DECODE:
			bcr.stopDecode();
			break;
		case STATE_SNAPSHOT:
			ret = STATE_IDLE;			
			break;
		default:
			ret = STATE_IDLE;			
		}
		return ret;
	}

	// ----------------------------------------
	public void onEvent(int event, int info, byte[] data, BarCodeReader reader)
	{
		switch (event)
		{
		case BarCodeReader.BCRDR_EVENT_SCAN_MODE_CHANGED:
			++modechgEvents;
			break;

		case BarCodeReader.BCRDR_EVENT_MOTION_DETECTED:
			++motionEvents;
			break;

		default:
			// process any other events here
			break;
		}
	}
	public boolean onKeyDown(int keyCode, KeyEvent event) {
	    switch (keyCode) {
	    	case KEY_SCAN:
	    		if(STATE_ISDECODING == false){
		    		decode_start = SystemClock.elapsedRealtime();
					doDecode();
					STATE_ISDECODING = true;
	    		}
	        	break;
	    }
	    
	    return super.onKeyDown(keyCode, event);
	}
	public boolean onKeyLongPress(int keyCode, KeyEvent event) {
	    switch (keyCode) {
	    	case KEY_SCAN:
	        	break;
	    }
	    
	    return super.onKeyLongPress(keyCode, event);
	}
	
}

